@@ -490,7 +490,7 @@ imxGetExpectationComponent <- function(model, component, defvar.row=1, subname=m
 		submNames <- sapply(strsplit(model$fitfunction$groups, ".", fixed=TRUE), "[", 1)
 		got <- list()
 		for(amod in submNames){
-			got[[amod]] <- imxGetExpectationComponent(model, component, defvar.row=1, subname=amod)
+			got[[amod]] <- imxGetExpectationComponent(model, component, defvar.row, subname=amod)
 		}
 		if(component=='vector' || tolower(component)=='standvector'){got <- unlist(got)}
 		got
@@ -657,12 +657,16 @@ generateNormalData <- function(model, nrows, subname, empirical, returnModel, us
 	# Check for definition variables
 	if(imxHasDefinitionVariable(model[[subname]])){
     if (is.null(origData)) {
-      stop("Definition variable(s) found, but no data is available")
+      stop("Definition variable(s) found, but no data are available")
     }
 		if (origData$type != 'raw') {
 			stop(paste("Definition variable(s) found, but original data is type",
 				omxQuotes(origData$type)))
 		}
+		#Trying to use .rmvnorm() with empirical=TRUE and nrow=1 leads to an R error raised in MASS::mvrnorm():
+		if(empirical){
+			stop("argument 'empirical=TRUE' to mxGenerateData() is not compatible with definition variables")
+		}
 		origData <- origData$observed
 		if(nrows != nrow(origData)){
 			stop("Definition variable(s) found, but the number of rows in the data do not match the number of rows requested for data generation.")
